#!/usr/local/bin/python
# @(#) $Id: acl-batch.py 834 2012-08-16 03:43:20Z leres $ (LBL)
"""acl-batch - batch drop/restore"""

from __future__ import print_function

import errno
import optparse
import os
import select
import socket
import sys

OPTS = None
PROG = '?'

DEFAULTSERVER = '127.0.0.1'
DEFAULTPORT = 1965
DEFAULTINFLIGHT = 100

# XXX redo with a class
IBUF = ''

def getacldline(s):
    """Get a line from the acld socket"""
    global IBUF

    while True:
	tup = IBUF.split('\r\n', 1)
	if len(tup) > 1:
	    IBUF = tup[1]
	    return tup[0]

	# Read more
	rlist = [s]
	wlist = []
	xlist = []
	timeout = 1.0
	(rlist, wlist, xlist) = select.select(rlist, wlist, xlist, timeout)

	# check for timeout?

	if s in rlist:
	    buf = s.recv(1024)
	    cc = len(buf)
	    if cc > 0:
		IBUF += buf

def getresponse(s, last_tup):
    """Reap response"""

    ret = True
    result = ''
    comment = ''
    what = last_tup[0]
    cookie = last_tup[1]
    if len(last_tup) > 2:
	target = last_tup[2]
    else:
	target = None

    line = getacldline(s)
    tup = line.split()
    n = len(tup)
    if n < 3 or n > 4 or tup[1] != cookie:
	print('%s: getresponse lose 1: "%s"' % (PROG, line), file=sys.stderr)
	sys.exit(1)

    # Save result
    extended_what = tup[2]

    # Read optional comment
    comments = []
    if n == 4:
	if tup[3] != '-':
	    print('%s: getresponse lose 2: "%s"' % (PROG, line),
		file=sys.stderr)
	    sys.exit(1)

	result = 'SUCCESS'
	while True:
	    line = getacldline(s)
	    if line == '.':
		break
	    comments.append(line)

    comment = ' '.join(comments)

    failed = '-failed'
    if extended_what.endswith(failed):
	extended_what = extended_what[:-len(failed)]
	ret = False
	result = 'FAILED'

    if what != extended_what:
	print('%s: getresponse lose 3: %s vs. %s (%s)' % (PROG,
	    what, extended_what, line), file=sys.stderr)
	sys.exit(1)

    # Optionally report transaction
    if OPTS.verbose or (not OPTS.quiet and what not in ('acld', 'exit')):
	parts = [tup[0], what]
	if target:
	    parts.append(target)
	if result:
	    parts.append(result + ':')
	if comment:
	    parts.append(comment)
	print(' '.join(parts))

    return ret

def sendline(s, tup):
    """Take a list, add \r\n's to each line and join"""
    buf = ''.join(map(lambda x: x + '\r\n', tup))
    s.send(buf)

def process(s, f):
    """Send drop/restore commands to ACLD on the socket"""
    inflight = []
    lastcookie = 0
    ret = 0

    if not getresponse(s, ['acld', str(lastcookie)]):
	print('%s: bad greeting from acld' % (PROG), file=sys.stderr)
	return 1

    # XXX make lastcookie == n (line number)>

    # Loop on addrs from the file
    n = 0
    for line in f.readlines():
	n += 1
	tup = line.rstrip().split(None, 2)
	if tup[0] not in ('drop', 'restore'):
	    print('# line %d: garbage "%s"' % (n, line.rstrip()),
		file=sys.stderr)
	    continue

	# Send the command
	lastcookie += 1
	tup.insert(1, str(lastcookie))
	inflight.append(tup[0:3])
	if len(tup) > 3:
	    comment = tup[-1]
	    tup[-1] = '-'
	else:
	    comment = ''

	sendline(s, (' '.join(tup),))
	if comment:
	    sendline(s, (comment, '.'))

	# Reap
	if len(inflight) >= OPTS.inflight:
	    if not getresponse(s, inflight.pop(0)):
		ret = 1

    while len(inflight) > 0:
	if not getresponse(s, inflight.pop(0)):
	    ret = 1

    lastcookie += 1
    inflight.append(['exit', str(lastcookie)])

    sendline(s, (' '.join(inflight[0]), ))
    if not getresponse(s, inflight.pop(0)):
	ret = 1

    return ret

def main(argv=None):
    """Parse options, output config"""
    global OPTS
    global PROG

    if not argv:
	argv = sys.argv

    PROG = os.path.basename(argv[0])
    usage = 'usage: %prog [-dqv] [-p PORT] [-s SERVER] [-i INFLIGHT] file'
    version = ' '.join([
	'acl-batch',
	'$Revision: 834 $'.lstrip('$').lstrip('Revision: ').rstrip('$'),
    ])

    parser = optparse.OptionParser(usage=usage, version=version)
    parser.add_option('-d', None,
        action='count', dest='debug', default=0,
        help='turn on debugging')
    parser.add_option('-q', None,
        action='count', dest='quiet', default=False,
        help='suppress per transaction messages')
    parser.add_option('-p', None,
        action='store', dest='port', type='int', default=DEFAULTPORT,
        help='port number (default: %default)')
    parser.add_option('-s', None,
        action='store', dest='server', default=DEFAULTSERVER,
        help='server address (default: %default)')
    parser.add_option('-i', None,
        action='store', dest='inflight', type='int', default=DEFAULTINFLIGHT,
        help='port number (default: %default)')

    parser.add_option('-v', None,
        action='store_true', dest='verbose', default=False,
        help='verbose messages')
    parser.add_option('', '--debugger',
        action='store_true', dest='Debug', default=False,
        help='interactive debugging')

    (OPTS, args) = parser.parse_args(args=argv[1:])

    if len(args) != 1:
	parser.print_help()
	return 1

    # Interactive debugging
    if OPTS.Debug:
        import pdb
        pdb.set_trace()

    fn = args.pop(0)
    try:
	s = socket.create_connection((OPTS.server, OPTS.port))
    except socket.error, e:
	if e.errno not in (errno.ENETUNREACH, errno.ECONNREFUSED):
	    raise e
	print('%s: create_connection: %s: %s' % (PROG, OPTS.server, e.strerror),
	    file=sys.stderr)
	return 1
    s.setblocking(0)
    try:
	with open(fn) as f:
	    ret = process(s, f)
    except IOError, e:
	if e.errno not in (errno.ENOENT, errno.EACCES, errno.EAGAIN):
	    raise e
	print('%s: open: %s: %s' % (PROG, fn, e.strerror), file=sys.stderr)
	if e.errno == errno.EAGAIN:
	    print('    (Lower -i for better results)', file=sys.stderr)
	ret = 1
    except KeyboardInterrupt:
	ret = 1

    s.close()
    return ret

if __name__ == "__main__":
    sys.exit(main())
